<?php declare(strict_types=1);

namespace App\Http\Requests\Admin\Projects\Documentations;

use App\Contracts\FormRequestDto;
use App\Dto\Service\Admin\Project\Documentation\StoreUpdate;
use App\Dto\Service\Admin\Project\DocumentationContent\Content;
use App\Dto\Service\Admin\Project\DocumentationContent\Contents;
use Illuminate\Foundation\Http\FormRequest;

final class StoreUpdateRequest extends FormRequest implements FormRequestDto
{
    /**
     * Get the validation rules that apply to the request.
     */
    public function rules(): array
    {
        return [
            'slug' => ['required', 'string', 'max:200', 'regex:/^[a-z0-9._-]+$/'],
            'is_public' => ['required', 'boolean'],
            'sort' => ['required', 'integer', 'min:-1000', 'max:1000'],
            'category_id' => ['nullable', 'integer', 'exists:documentation_categories,id'],
            'content.*.title' => ['required', 'string', 'max:255'],
            'content.*.content' => ['nullable', 'string'],
        ];
    }

    public function getDto(): StoreUpdate
    {
        $categoryId = $this->input('category_id', null);
        if (!\is_null($categoryId)) {
            $categoryId = (int) $categoryId;
        }

        return new StoreUpdate(
            slug: $this->input('slug'),
            isPublic: (bool) $this->input('is_public', false),
            sort: (int) $this->input('sort'),
            contents: $this->getContents(),
            categoryId: $categoryId,
        );
    }

    private function getContents(): Contents
    {
        $contents = new Contents();
        foreach ($this->input('content', []) as $languageId => $content) {
            $contents->addContent(new Content(
                languageId: (int) $languageId,
                title:      $content['title'],
                content:    $content['content'] ?? '',
            ));
        }
        return $contents;
    }
}
